"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-lonely-if */
/* eslint-disable @typescript-eslint/no-var-requires */
const fs_1 = __importDefault(require("fs"));
const ora_1 = __importDefault(require("ora"));
const child_process_1 = require("child_process");
const dependencies_1 = require("../config/dependencies");
const misc_1 = require("../config/misc");
const index_eslint_1 = __importDefault(require("../config/eslint/index.eslint"));
class PackageManager {
    static initPackage(projectName, pkgManager, pkgQuestions) {
        const packageSpinner = ora_1.default(`🔨 Initializing The Package with ${pkgManager}...`).start();
        fs_1.default.mkdirSync(projectName);
        if (pkgQuestions === 'Go with defaults') {
            child_process_1.execSync(`cd ${projectName} && ${pkgManager} init -y`, {
                stdio: 'ignore',
            });
        }
        else {
            child_process_1.execSync(`cd ${projectName} && ${pkgManager} init`, { stdio: 'inherit' });
        }
        packageSpinner.succeed(`🔨 Initialized The Package with ${pkgManager}`);
    }
    static createScripts(projectName, typeScript, babel, extraLibs, tests) {
        const scriptsSpinner = ora_1.default('📜 Creating Scripts...').start();
        const pkgJSON = JSON.parse(fs_1.default.readFileSync(`${projectName}/package.json`, 'utf8'));
        pkgJSON.scripts = {};
        if (typeScript) {
            pkgJSON.scripts.start = 'npx ts-node src/index.ts';
        }
        else if (babel) {
            pkgJSON.scripts.start = 'npx babel-node src/index.js';
        }
        else {
            pkgJSON.scripts.start = 'node src/index.js';
        }
        typeScript ? (pkgJSON.main = 'index.ts') : (pkgJSON.main = 'index.js');
        if (typeScript) {
            pkgJSON.scripts['start:source'] = 'npx tsc && node out/index.ts';
            pkgJSON.scripts.build = 'npx tsc';
            pkgJSON.scripts.watch = 'npx tsc src/index.ts --watch';
        }
        if (babel) {
            pkgJSON.scripts['start:source'] = 'npm run build && node out/index.js';
            pkgJSON.scripts.build =
                'npx babel src -d out --presets @babel/preset-env';
            pkgJSON.scripts.watch =
                'npx babel src -d out --presets @babel/preset-env -w';
        }
        if (tests === 'Jest') {
            pkgJSON.scripts.test = 'npx jest';
        }
        if (tests === 'Mocha + Chai') {
            if (typeScript) {
                pkgJSON.scripts.test =
                    'npx mocha __tests__/**/*.ts -r ts-node/register';
            }
            else if (babel) {
                pkgJSON.scripts.test =
                    'npx mocha __tests__/**/*.js --require @babel/register --require @babel/polyfill';
            }
            else {
                pkgJSON.scripts.test = 'npx mocha __tests__/**/*.js';
            }
        }
        if (extraLibs.includes('ESLint')) {
            typeScript
                ? (pkgJSON.scripts.lint = 'npx eslint src/*.ts')
                : (pkgJSON.scripts.lint = 'npx eslint src/*.js');
            typeScript
                ? (pkgJSON.scripts['lint:fix'] = 'npx eslint src/*.ts --fix')
                : (pkgJSON.scripts['lint:fix'] = 'npx eslint src/*.js --fix');
        }
        if (extraLibs.includes('Prettier')) {
            typeScript
                ? (pkgJSON.scripts.format = 'npx prettier src/*.ts --write')
                : (pkgJSON.scripts.format = 'npx prettier src/*.js --write');
        }
        if (extraLibs.includes('nodemon or ts-node-dev')) {
            if (typeScript) {
                pkgJSON.scripts.dev = 'npx ts-node-dev --respawn src/index';
            }
            else if (babel) {
                pkgJSON.scripts.dev = 'npx nodemon --exec babel-node src/index';
            }
            else {
                pkgJSON.scripts.dev = 'npx nodemon src/index';
            }
        }
        fs_1.default.writeFileSync(`${projectName}/package.json`, JSON.stringify(pkgJSON, null, 2));
        scriptsSpinner.succeed('📜 Created Scripts');
    }
    static addPackageDetails(projectName, licenseType, licenseAuthor, git, hostingPLatform, platformUsername, repositoryName) {
        const detailsSpinner = ora_1.default('📋 Adding Package Details...').start();
        const pkgJSON = JSON.parse(fs_1.default.readFileSync(`${projectName}/package.json`, 'utf8'));
        pkgJSON.description = 'YOUR DESCRIPTION';
        pkgJSON.license = licenseType || 'YOUR LICENSE';
        pkgJSON.author = licenseAuthor || 'YOUR NAME <YOUR EMAIL>';
        pkgJSON.keywords = ['key', 'words'];
        if (git) {
            pkgJSON.repository = {
                type: 'git',
                url: `https://${hostingPLatform.toLowerCase()}.com/${platformUsername}/${repositoryName}`,
            };
            pkgJSON.bugs = {
                url: `https://${hostingPLatform.toLowerCase()}.com/${platformUsername}/${repositoryName}/issues`,
            };
        }
        fs_1.default.writeFileSync(`${projectName}/package.json`, JSON.stringify(pkgJSON, null, 2));
        detailsSpinner.succeed('📋 Added Package Details');
    }
    static installTsDependencies(pkgManager, projectName) {
        const tsSpinner = ora_1.default('📥 Setting Up TypeScript...').start();
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.tsDependencies}`, {
                stdio: 'ignore',
            });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.tsDependencies}`, {
                stdio: 'ignore',
            });
        }
        tsSpinner.succeed('📥 Set Up TypeScript');
    }
    static addBabel(projectName, pkgManager) {
        const babelSpinner = ora_1.default('🐠 Adding Babel...').start();
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.babelDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.babelDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        fs_1.default.writeFileSync(`${projectName}/.babelrc`, misc_1.babelConfig);
        babelSpinner.succeed('🐠 Added Babel');
    }
    static addEslint(projectName, pkgManager, eslintConfig, typeScript, babel, prettier) {
        const linterSpinner = ora_1.default('🔎 Adding ESLint...').start();
        fs_1.default.writeFileSync(`${projectName}/.eslintignore`, misc_1.eslintIgnore);
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.eslintDependencies} -D`, {
                stdio: 'ignore',
            });
            if (typeScript) {
                child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.eslintTsDependencies} -D`, {
                    stdio: 'ignore',
                });
            }
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.eslintDependencies} -D`, {
                stdio: 'ignore',
            });
            if (typeScript) {
                child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.eslintTsDependencies} -D`, {
                    stdio: 'ignore',
                });
            }
        }
        switch (eslintConfig) {
            case 'Only Errors':
                if (typeScript && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.tsPretty, null, 2));
                }
                else if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.ts, null, 2));
                }
                else if (babel && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.babelPretty, null, 2));
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.babel, null, 2));
                }
                else if (prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.jsPretty, null, 2));
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.js, null, 2));
                }
                break;
            case 'AirBNB':
                if (typeScript && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.tsPretty, null, 2));
                }
                else if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.ts, null, 2));
                }
                else if (babel && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.babelPretty, null, 2));
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.babel, null, 2));
                }
                else if (prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.jsPretty, null, 2));
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.airBnb.js, null, 2));
                }
                if (pkgManager === 'npm') {
                    child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.lintAirBnbDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                if (pkgManager === 'yarn') {
                    child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.lintAirBnbDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                break;
            case 'Google':
                if (typeScript && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.tsPretty, null, 2));
                }
                else if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.ts, null, 2));
                }
                else if (babel && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.babelPretty, null, 2));
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.babel, null, 2));
                }
                else if (prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.jsPretty, null, 2));
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.google.js, null, 2));
                }
                if (pkgManager === 'npm') {
                    child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.lintGoogleDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                if (pkgManager === 'yarn') {
                    child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.lintGoogleDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                break;
            case 'Standard':
                if (typeScript && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.tsPretty, null, 2));
                }
                else if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.ts, null, 2));
                }
                else if (babel && prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.babelPretty, null, 2));
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.babel, null, 2));
                }
                else if (prettier) {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.jsPretty, null, 2));
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/.eslintrc.json`, JSON.stringify(index_eslint_1.default.standard.js, null, 2));
                }
                if (pkgManager === 'npm') {
                    child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.lintStandardDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                if (pkgManager === 'yarn') {
                    child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.lintStandardDependencies} -D`, {
                        stdio: 'ignore',
                    });
                }
                break;
        }
        linterSpinner.succeed('🔎 Added ESLint');
    }
    static addPrettier(projectName, pkgManager) {
        const prettierSpinner = ora_1.default('🧹 Adding Prettier...').start();
        fs_1.default.writeFileSync(`${projectName}/.prettierignore`, misc_1.prettierIgnore);
        fs_1.default.writeFileSync(`${projectName}/.prettierrc`, misc_1.prettierConfig);
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i prettier -D`, { stdio: 'ignore' });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add prettier -D`, {
                stdio: 'ignore',
            });
        }
        prettierSpinner.succeed('🧹 Added Prettier');
    }
    static attachLinterWithPrettier(projectName, pkgManager) {
        const linterPrettierSpinner = ora_1.default('🖇 Connecting ESLint with Prettier...').start();
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.lintFormatDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.lintFormatDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        linterPrettierSpinner.succeed('🖇 Connected ESLint with Prettier');
    }
    static attachLinterWithBabel(projectName, pkgManager) {
        const linterPrettierSpinner = ora_1.default('🖇 Connecting ESLint with Babel...').start();
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i ${dependencies_1.babelLinterDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add ${dependencies_1.babelLinterDependencies} -D`, {
                stdio: 'ignore',
            });
        }
        linterPrettierSpinner.succeed('🖇 Connected ESLint with Babel');
    }
    static addDotenv(projectName, pkgManager, typeScript, babel) {
        const dotenvSpinner = ora_1.default('🔒 Adding Dotenv...').start();
        if (pkgManager === 'npm') {
            child_process_1.execSync(`cd ${projectName} && npm i dotenv -D`, {
                stdio: 'ignore',
            });
        }
        if (pkgManager === 'yarn') {
            child_process_1.execSync(`cd ${projectName} && yarn add dotenv -D`, {
                stdio: 'ignore',
            });
        }
        fs_1.default.writeFileSync(`${projectName}/.env`, misc_1.dotenvFile);
        if (typeScript) {
            fs_1.default.appendFileSync(`${projectName}/src/index.ts`, misc_1.dotenvESFile);
        }
        else if (babel) {
            fs_1.default.appendFileSync(`${projectName}/src/index.js`, misc_1.dotenvESFile);
        }
        else {
            fs_1.default.appendFileSync(`${projectName}/src/index.js`, misc_1.dotenvCommonFile);
        }
        dotenvSpinner.succeed('🔒 Added Dotenv');
    }
    static addChangesMonitor(projectName, pkgManager, typeScript) {
        const nodemonSpinner = ora_1.default('🔁 Adding Changes Montitor...').start();
        if (pkgManager === 'npm') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && npm i ts-node-dev -D`, {
                    stdio: 'ignore',
                })
                : child_process_1.execSync(`cd ${projectName} && npm i nodemon -D`, {
                    stdio: 'ignore',
                });
        }
        if (pkgManager === 'yarn') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && yarn add ts-node-dev -D`, {
                    stdio: 'ignore',
                })
                : child_process_1.execSync(`cd ${projectName} && yarn add nodemon -D`, {
                    stdio: 'ignore',
                });
        }
        nodemonSpinner.succeed('🔁 Added Changes Monitor');
    }
    static addJest(projectName, pkgManager, typeScript, babel) {
        const jestSpinner = ora_1.default('🃏 Adding Jest...').start();
        if (pkgManager === 'npm') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && npm i jest @types/jest ts-jest -D`, {
                    stdio: 'ignore',
                })
                : child_process_1.execSync(`cd ${projectName} && npm i jest -D`, {
                    stdio: 'ignore',
                });
            if (babel) {
                child_process_1.execSync(`cd ${projectName} && npm i jest-babel -D`, {
                    stdio: 'ignore',
                });
            }
        }
        if (pkgManager === 'yarn') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && yarn add jest @types/jest ts-jest -D`, { stdio: 'ignore' })
                : child_process_1.execSync(`cd ${projectName} && yarn add jest -D`, {
                    stdio: 'ignore',
                });
            if (babel) {
                child_process_1.execSync(`cd ${projectName} && yarn add jest-babel -D`, {
                    stdio: 'ignore',
                });
            }
        }
        if (typeScript) {
            child_process_1.execSync(`cd ${projectName} && npx ts-jest config:init`, {
                stdio: 'ignore',
            });
            fs_1.default.renameSync(`${projectName}/jest.config.js`, `${projectName}/jest.config.ts`);
        }
        jestSpinner.succeed('🃏 Added Jest');
    }
    static addMochaChai(projectName, pkgManager, typeScript, babel) {
        const mochaChaiSpinner = ora_1.default('🍵 Adding Mocha & Chai...').start();
        if (pkgManager === 'npm') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && npm i mocha chai @types/mocha @types/chai -D`, {
                    stdio: 'ignore',
                })
                : child_process_1.execSync(`cd ${projectName} && npm i mocha chai -D`, {
                    stdio: 'ignore',
                });
            if (babel) {
                child_process_1.execSync(`cd ${projectName} && npm i @babel/register @babel/polyfill -D`, {
                    stdio: 'ignore',
                });
            }
        }
        if (pkgManager === 'yarn') {
            typeScript
                ? child_process_1.execSync(`cd ${projectName} && yarn add mocha chai @types/mocha @types/chai -D`, {
                    stdio: 'ignore',
                })
                : child_process_1.execSync(`cd ${projectName} && yarn add mocha chai -D`, {
                    stdio: 'ignore',
                });
            if (babel) {
                child_process_1.execSync(`cd ${projectName} && yarn add @babel/register @babel/polyfill -D`, {
                    stdio: 'ignore',
                });
            }
        }
        mochaChaiSpinner.succeed('🍵 Added Mocha & Chai');
    }
}
exports.default = PackageManager;
