"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const os_1 = require("os");
const path_1 = __importDefault(require("path"));
const ora_1 = __importDefault(require("ora"));
const chalk_1 = __importDefault(require("chalk"));
const license_1 = require("license");
const child_process_1 = require("child_process");
const package_manager_1 = __importDefault(require("./package_manager"));
const misc_1 = require("../config/misc");
const ts_config_1 = __importDefault(require("../config/ts_config"));
class ProjectGenerator {
    static handleProjectSettings(projectName, pkgManager, pkgQuestions, typeScript, babel, eslintConfig, extraLibs, extraOptions, tests, licenseType, licenseAuthor, hostingPlatform, platformUsername, repositoryName, savePreset, presetName) {
        package_manager_1.default.initPackage(projectName, pkgManager, pkgQuestions);
        if (typeScript) {
            package_manager_1.default.installTsDependencies(pkgManager, projectName);
            ProjectGenerator.initTypeScript(projectName);
        }
        if (babel) {
            package_manager_1.default.addBabel(projectName, pkgManager);
        }
        if (extraOptions) {
            if (extraOptions.includes('git')) {
                ProjectGenerator.initGit(projectName, hostingPlatform, platformUsername, repositoryName);
            }
            if (extraOptions.includes('README')) {
                ProjectGenerator.addReadme(projectName);
            }
            if (extraOptions.includes('LICENSE')) {
                ProjectGenerator.addLicense(projectName, licenseType, licenseAuthor);
            }
        }
        ProjectGenerator.createSourceFolder(projectName, typeScript, babel);
        package_manager_1.default.addPackageDetails(projectName, licenseType, licenseAuthor, extraOptions.includes('git'), hostingPlatform, platformUsername, repositoryName);
        package_manager_1.default.createScripts(projectName, typeScript, babel, extraLibs, tests);
        if (tests === 'Jest') {
            ProjectGenerator.createTestsFolder(projectName, typeScript, babel, tests);
            package_manager_1.default.addJest(projectName, pkgManager, typeScript, babel);
        }
        if (tests === 'Mocha + Chai') {
            ProjectGenerator.createTestsFolder(projectName, typeScript, babel, tests);
            package_manager_1.default.addMochaChai(projectName, pkgManager, typeScript, babel);
        }
        if (extraLibs.includes('Prettier')) {
            package_manager_1.default.addPrettier(projectName, pkgManager);
        }
        if (extraLibs.includes('ESLint') && extraLibs.includes('Prettier')) {
            package_manager_1.default.attachLinterWithPrettier(projectName, pkgManager);
        }
        if (extraLibs.includes('ESLint') && babel) {
            package_manager_1.default.attachLinterWithBabel(projectName, pkgManager);
        }
        if (extraLibs.includes('ESLint')) {
            package_manager_1.default.addEslint(projectName, pkgManager, eslintConfig, typeScript, babel, extraLibs.includes('Prettier'));
        }
        if (extraLibs.includes('dotenv')) {
            package_manager_1.default.addDotenv(projectName, pkgManager, typeScript, babel);
        }
        if (extraLibs.includes('nodemon or ts-node-dev')) {
            package_manager_1.default.addChangesMonitor(projectName, pkgManager, typeScript);
        }
        if (savePreset) {
            ProjectGenerator.savePreset(presetName, {
                projectName,
                pkgManager,
                pkgQuestions,
                typeScript,
                babel,
                eslintConfig,
                extraLibs,
                extraOptions,
                tests,
                licenseType,
                licenseAuthor,
                hostingPlatform,
                platformUsername,
                repositoryName,
            });
        }
        console.log(chalk_1.default.greenBright('🎉 Ready!'));
        console.log(chalk_1.default.greenBright(`🚀 cd ${projectName} && ${pkgManager} start`));
        if (extraOptions.includes('git')) {
            console.log(chalk_1.default.redBright('Dont forget to create repo at:'));
            console.log(chalk_1.default.cyanBright(`https://${hostingPlatform.toLowerCase()}.com/${platformUsername}/${repositoryName}`));
            console.log(chalk_1.default.redBright('Do not forget to push your files to the repo'));
            console.log(chalk_1.default.cyanBright('git add . && git commit -m "Initial commit"'));
            console.log(chalk_1.default.cyanBright('git push origin master'));
        }
    }
    static initTypeScript(projectName) {
        child_process_1.execSync(`cd ${projectName} && npx tsc --init`, { stdio: 'ignore' });
        fs_1.default.writeFileSync(`${projectName}/tsconfig.json`, JSON.stringify(ts_config_1.default, null, 2));
    }
    static createSourceFolder(projectName, typeScript, babel) {
        const srcSpinner = ora_1.default('📂 Creating Source Folder...').start();
        fs_1.default.mkdirSync(`${projectName}/src`, { recursive: true });
        typeScript
            ? fs_1.default.writeFileSync(`${projectName}/src/index.ts`, misc_1.defaultTsFile)
            : fs_1.default.writeFileSync(`${projectName}/src/index.js`, misc_1.defaultJsFile);
        if (babel) {
            fs_1.default.writeFileSync(`${projectName}/src/index.js`, misc_1.defaultBabelFile);
        }
        srcSpinner.succeed('📂 Created Source Folder');
    }
    static createTestsFolder(projectName, typeScript, babel, testingFramework) {
        const testsSpinner = ora_1.default('📂 Creating Tests Folder...').start();
        fs_1.default.mkdirSync(`${projectName}/__tests__`, { recursive: true });
        switch (testingFramework) {
            case 'Jest':
                if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.test.ts`, misc_1.jestESFile);
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.test.js`, misc_1.jestESFile);
                    fs_1.default.writeFileSync(`./${projectName}/jest.config.js`, misc_1.jestConfig);
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.test.js`, misc_1.jestCommonFile);
                    fs_1.default.writeFileSync(`${projectName}/jest.config.js`, misc_1.jestConfig);
                }
                break;
            case 'Mocha + Chai':
                if (typeScript) {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.spec.ts`, misc_1.mochaChaiESFile);
                }
                else if (babel) {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.spec.js`, misc_1.mochaChaiESFile);
                }
                else {
                    fs_1.default.writeFileSync(`${projectName}/__tests__/index.spec.js`, misc_1.mochaChaiCommonFile);
                }
        }
        testsSpinner.succeed('📂 Created Tests Folder');
    }
    static initGit(projectName, hostingPlatform, platformUsername, repositoryName) {
        const gitSpinner = ora_1.default('📚 Initializing git...').start();
        child_process_1.execSync(`cd ${projectName} && git init`, { stdio: 'ignore' });
        switch (hostingPlatform) {
            case 'GitHub':
                child_process_1.execSync(`cd ${projectName} && git remote add origin https://github.com/${platformUsername}/${repositoryName}.git`, { stdio: 'ignore' });
                break;
            case 'GitLab':
                child_process_1.execSync(`cd ${projectName} && git remote add origin https://gitlab.com/${platformUsername}/${repositoryName}.git`, { stdio: 'ignore' });
                break;
        }
        child_process_1.execSync(`cd ${projectName} && npx gitignore node`, { stdio: 'ignore' });
        gitSpinner.succeed('📚 Initialized git');
    }
    static addReadme(projectName) {
        const readmeSpinner = ora_1.default('📄 Creating README...').start();
        fs_1.default.writeFileSync(`${projectName}/README.md`, `# ${projectName}`);
        readmeSpinner.succeed('📄 Created README');
    }
    static addLicense(projectName, licenseType, licenseAuthor) {
        const licenseSpinner = ora_1.default('📜 Creating License...').start();
        switch (licenseType) {
            case 'MIT':
                fs_1.default.writeFileSync(`${projectName}/LICENSE`, license_1.getLicense('MIT', {
                    author: licenseAuthor || '<your name>',
                    year: `${new Date().getFullYear()}`,
                }));
                break;
            case 'Apache-2.0':
                fs_1.default.writeFileSync(`${projectName}/LICENSE`, license_1.getLicense('Apache-2.0', {
                    author: licenseAuthor || '<your name>',
                    year: `${new Date().getFullYear()}`,
                }));
                break;
            case 'GPL-3.0':
                fs_1.default.writeFileSync(`${projectName}/LICENSE`, license_1.getLicense('GPL-3.0', {
                    author: licenseAuthor || '<your name>',
                    year: `${new Date().getFullYear()}`,
                }));
                break;
            case 'ISC':
                fs_1.default.writeFileSync(`${projectName}/LICENSE`, license_1.getLicense('ISC', {
                    author: licenseAuthor || '<your name>',
                    year: `${new Date().getFullYear()}`,
                }));
                break;
            case 'BSD-3-Clause':
                fs_1.default.writeFileSync(`${projectName}/LICENSE`, license_1.getLicense('BSD-3-Clause', {
                    author: licenseAuthor || '<your name>',
                    year: `${new Date().getFullYear()}`,
                }));
                break;
        }
        licenseSpinner.succeed('📜 Created License');
    }
    static savePreset(presetName, settings) {
        const savePresetSpinner = ora_1.default('💾 Saving preset...').start();
        const { projectName, pkgManager, pkgQuestions, typeScript, babel, eslintConfig, extraLibs, extraOptions, tests, licenseType, licenseAuthor, hostingPlatform, platformUsername, repositoryName, } = settings;
        if (!fs_1.default.existsSync(path_1.default.join(os_1.homedir(), '.forge-node-app-rc'))) {
            fs_1.default.writeFileSync(path_1.default.join(os_1.homedir(), '.forge-node-app-rc'), '{\n}');
        }
        const presetsConfig = JSON.parse(fs_1.default.readFileSync(`${path_1.default.join(os_1.homedir())}/.forge-node-app-rc`, 'utf-8'));
        presetsConfig[presetName] = {
            projectName,
            pkgManager,
            pkgQuestions,
            typeScript,
            babel,
            eslintConfig,
            extraLibs,
            extraOptions,
            tests,
            licenseType,
            licenseAuthor,
            hostingPlatform,
            platformUsername,
            repositoryName,
        };
        fs_1.default.writeFileSync(path_1.default.join(os_1.homedir(), '.forge-node-app-rc'), JSON.stringify(presetsConfig, null, 2));
        savePresetSpinner.succeed(`💾 Saved preset: ${presetName}`);
    }
    static loadPresets() {
        if (!fs_1.default.existsSync(path_1.default.join(os_1.homedir(), '.forge-node-app-rc'))) {
            fs_1.default.writeFileSync(path_1.default.join(os_1.homedir(), '.forge-node-app-rc'), '{\n}');
        }
        const presets = JSON.parse(fs_1.default.readFileSync(`${path_1.default.join(os_1.homedir())}/.forge-node-app-rc`, 'utf-8'));
        return Object.keys(presets);
    }
    static handleProjectWithCustomPreset(presetName) {
        const presets = JSON.parse(fs_1.default.readFileSync(`${path_1.default.join(os_1.homedir())}/.forge-node-app-rc`, 'utf-8'));
        ProjectGenerator.handleProjectSettings(presets[presetName].projectName, presets[presetName].pkgManager, presets[presetName].pkgQuestions, presets[presetName].typeScript, presets[presetName].babel, presets[presetName].eslintConfig, presets[presetName].extraLibs, presets[presetName].extraOptions, presets[presetName].tests, presets[presetName].licenseType, presets[presetName].licenseAuthor, presets[presetName].hostingPlatform, presets[presetName].platformUsername, presets[presetName].repositoryName);
        console.log(`🔧 Generated app using ${presetName} preset`);
    }
    static handleProjectWithDefaultPreset(presetName) {
        switch (presetName) {
            case 'Default (yarn, TypeScript, ESLint (Errors only), Jest)':
                ProjectGenerator.handleProjectSettings('node-app', 'yarn', 'Go with defaults', true, false, 'Only Errors', ['ESLint', 'Unit Tests'], undefined, 'Jest');
                break;
            case 'Default (npm, JavaScript, ESLint (AirBNB), Mocha + Chai)':
                ProjectGenerator.handleProjectSettings('node-app', 'npm', 'Go with defaults', false, false, 'AirBNB', ['ESLint', 'Unit Tests'], undefined, 'Mocha + Chai');
                break;
            case 'Default (npm, Babel, ESLint (AirBNB), nodemon)':
                ProjectGenerator.handleProjectSettings('node-app', 'npm', 'Go with defaults', false, true, 'AirBNB', ['ESLint', 'Unit Tests', 'nodemon or ts-node-dev'], undefined);
                break;
        }
        console.log(`🔧 Generated app using ${presetName} preset`);
    }
}
exports.default = ProjectGenerator;
